using RaftModLoader;
using UnityEngine;
using HMLLibrary;
using HarmonyLib;
using System.Collections.Generic;
using System.Reflection;
using Sirenix.Utilities;

public class NoShimmer : Mod
{
    public static NoShimmer instance;
    public Harmony harmonyInstance;
    public string harmonyId = "fr.tek.noshimmer";

    public Dictionary<Material, float> fixedMaterials = new Dictionary<Material, float>();
    bool initialized;
    public void Start()
    {
        instance = this;
        harmonyInstance = new Harmony(harmonyId);
        harmonyInstance.PatchAll(Assembly.GetExecutingAssembly());
        Renderer[] renderers = FindObjectsOfType<Renderer>();
        renderers.ForEach(renderer =>
        {
            RemoveShimmer(renderer.gameObject);
        });
        Debug.Log("Mod NoShimmer has been loaded!");
        initialized = true;
    }

    public override bool CanUnload(ref string message)
    {
        if (!initialized)
            message = "The mod is still loading !";
        return initialized;
    }

    public void OnModUnload()
    {
        harmonyInstance.UnpatchAll(harmonyId);
        fixedMaterials.ForEach(m =>
        {
            if (m.Key != null)
            {
                m.Key?.SetFloat("_Shimmer", m.Value);
                Debug.Log("Unfixed material " + m.Key?.name);
            }
        });
        Debug.Log("Mod NoShimmer has been unloaded!");
    }

    public static void RemoveShimmer(GameObject obj)
    {
        if (!obj) return;
        List<Material> m = GetAllMaterials(obj);
        m.ForEach(material =>
        {
            if (material.HasProperty("_Shimmer"))
            {
                if (!instance.fixedMaterials.ContainsKey(material))
                {
                    Debug.Log("Fixed material " + material.name);
                    instance.fixedMaterials.Add(material, material.GetFloat("_Shimmer"));
                    material.SetFloat("_Shimmer", 0f);
                }
            }
        });
    }

    public static List<Material> GetAllMaterials(GameObject obj)
    {
        List<Material> materials = new List<Material>();

        // Get materials from the current GameObject
        Renderer renderer = obj.GetComponent<Renderer>();
        if (renderer != null)
        {
            materials.AddRange(renderer.materials);
        }

        // Get materials from children recursively
        foreach (Transform child in obj.transform)
        {
            materials.AddRange(GetAllMaterials(child.gameObject));
        }

        return materials;
    }
}

[HarmonyPatch(typeof(PickupItem), "Start")]
static class Patch_NoShimmer_1
{
    static void Postfix(PickupItem __instance)
    {
        NoShimmer.RemoveShimmer(__instance.gameObject);
    }
}

[HarmonyPatch(typeof(CarryNetworked), "Start")]
static class Patch_NoShimmer_2
{
    static void Postfix(CarryNetworked __instance)
    {
        NoShimmer.RemoveShimmer(__instance.gameObject);
    }
}